/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */

package org.objectweb.dream.cache.lib;

import org.objectweb.dream.cache.api.UnFixProtocolException;
import org.objectweb.dream.cache.replacement.api.ReplaceableCacheEntry;

/**
 * This class is a simple implementation of the ReplaceableCacheEntry interface.
 * 
 * @author S.Chassande-Barrioz, E.Bruneton
 */
public class BasicReplaceableCacheEntry implements ReplaceableCacheEntry {
    Object object;

    Object id;

    int users;

    long age;

    protected BasicReplaceableCacheEntry(Object id) {
        this.id = id;
        this.object = this;
    }

    public BasicReplaceableCacheEntry(Object id, Object object) {
        this.object = object;
        this.id = id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.dream.cache.api.CacheEntry#getCeObject()
     */
    public Object getCeObject() {
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.dream.cache.api.CacheEntry#getCeIdentifier()
     */
    public Object getCeIdentifier() {
        return id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.dream.cache.api.FixableCacheEntry#fixCe()
     */
    public void fixCe() {
        users++;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.dream.cache.api.FixableCacheEntry#unfixCe()
     */
    public void unfixCe() throws UnFixProtocolException {
        if (users == 0) {
            throw new UnFixProtocolException();
        }
        users--;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.dream.cache.api.FixableCacheEntry#getCeFixCount()
     */
    public int getCeFixCount() {
        return users;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.dream.cache.replacement.api.ReplaceableCacheEntry#getCeAge
     * ()
     */
    public long getCeAge() {
        return age;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.dream.cache.replacement.api.ReplaceableCacheEntry#setCeAge
     * (long)
     */
    public void setCeAge(long age) {
        this.age = age;
    }
}
