/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.factory;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.factory.ChainedInstantiationException;

/**
 * Mixin which adds created component as sub component of the component this
 * controller belong.
 */
public abstract class ContentFactoryMixin
{

  /**
   * Private constructor
   *
   */
  private ContentFactoryMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.factory.GenericFactory#newFcInstance(Type,
   *      Object, Object)
   */
  public Component newFcInstance(final Type type, final Object controllerDesc,
      final Object contentDesc) throws InstantiationException
  {
    Component component = _super_newFcInstance(type, controllerDesc,
        contentDesc);
    try
    {
      _this_weaveableCC.addFcSubComponent(component);
    }
    catch (IllegalContentException e)
    {
      throw new ChainedInstantiationException(e, _this_weaveableC,
          "Unable to add created component has factory sub component");
    }
    catch (IllegalLifeCycleException e)
    {
      // cannot happened
      throw new InternalError();
    }
    return component;
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The <tt>weaveableC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component}interface of the component to
   * which this controller object belongs.
   */

  public Component         _this_weaveableC;

  /**
   * The <tt>weaveableCC</tt> field required by this mixin. This field is
   * supposed to reference the {@link ContentController}interface of the
   * component to which this controller object belongs.
   */
  public ContentController _this_weaveableCC;

  /**
   * The
   * {@link org.objectweb.fractal.api.factory.GenericFactory#newFcInstance newFcInstance}
   * method overriden by this mixin.
   * 
   * @see org.objectweb.fractal.api.factory.GenericFactory#newFcInstance
   *      newFcInstance
   */

  public abstract Component _super_newFcInstance(Type type,
      Object controllerDesc, Object contentDesc) throws InstantiationException;

}