/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.lifecycle;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;

/**
 * The run method call {@link LifeCycleCoordinator#setFcStarted()} and then the
 * {@link ASyncStarterCoordinator#started}
 */
public class AsyncStarter implements Runnable
{
  private LifeCycleCoordinator    lc;
  private ASyncStarterCoordinator coordinator;

  /**
   * @param lc the {@link LifeCycleCoordinator lifecycle} to start
   * @param coordinator the {@link ASyncStarterCoordinator coordinator} to
   *          notify when the component is started.
   */
  public AsyncStarter(LifeCycleCoordinator lc,
      ASyncStarterCoordinator coordinator)
  {
    this.lc = lc;
    this.coordinator = coordinator;
  }

  /**
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    try
    {
      lc.setFcStarted();
      coordinator.started(lc, null);
    }
    catch (IllegalLifeCycleException e)
    {
      coordinator.started(lc, e);
    }
  }

}
