/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import org.objectweb.dream.pool.ObjectPoolItf;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.pool.RecyclableFactoryType;

class BasicChunkFactoryPoolingImpl<T extends AbstractChunk<T>>
    extends
      ChunkFactoryReference<T> implements RecyclableFactoryType
{
  private final Class<T>      chunkClass;
  private final T             chunkInstance;
  private final ObjectPoolItf chunkPool;

  /**
   * Constructor
   * 
   * @param chunkClass class defining the chunk type
   * @param owner {@link MessageManager} owner of the chunk
   * @param chunkPool {@link ObjectPoolItf} pool of chunk associated to this
   *          factory
   * @throws Exception
   */
  BasicChunkFactoryPoolingImpl(final Class<T> chunkClass,
      final MessageManagerType owner, final ObjectPoolItf chunkPool)
      throws Exception
  {
    super(owner);
    this.chunkClass = chunkClass;
    this.chunkPool = chunkPool;
    chunkInstance = chunkClass.newInstance();
  }

  /**
   * Creates a new chunk of type {@link AbstractChunk<T>} via the pool
   */
  @SuppressWarnings("unchecked")
  protected T createChunk()
  {
    return (T) chunkPool.newInstance();
  }

  /**
   * @see org.objectweb.dream.message.ChunkFactoryReference#recycleChunk(org.objectweb.dream.message.AbstractChunk)
   */
  protected void recycleChunk(final T chunk)
  {
    chunk.recycle();
    chunkPool.recycleInstance(chunk);
  }

  /**
   * Get chunk class associated to this factory
   * 
   * @return a class of type {@link T}
   */
  final Class<T> getChunkClass()
  {
    return chunkClass;
  }

  /**
   * @see org.objectweb.dream.pool.RecyclableFactoryType#newInstance()
   */
  public Recyclable newInstance()
  {
    AbstractChunk<T> newChunk = chunkInstance.newChunk();
    newChunk.initAbstractChunk(this, getOwner());
    return newChunk;
  }
}
