/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Pierre Garcia
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Basic message codec that create empty {@link Message}.
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class MessageCreatorBasic extends AbstractDelegatingMessageCodec
{
  @Requires(name = MessageManagerType.ITF_NAME)
  protected MessageManagerType messageManagerItf;
  
  // ---------------------------------------------------------------------------
  // Implementation of the MessageCodec interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageCodec#encode(CodecInputOutput, Message) warning : this
   *      implementation does nothing
   */
  public void encode(CodecInputOutput cio, Message message) throws IOException
  {
    if (this.delegateMessageCodecItf != null)
    {
      this.delegateMessageCodecItf.encode(cio, message);
    }
    // do nothing
  }

  /**
   * @see MessageCodec#decode(CodecInputOutput)
   */
  public Message decode(CodecInputOutput cio) throws IOException
  {
    if (this.delegateMessageCodecItf != null)
    {
      this.delegateMessageCodecItf.decode(cio);
    }
    return messageManagerItf.createMessage();
  }
}
