/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.ref.Reference;
import java.net.Socket;

import org.objectweb.dream.pool.Recyclable;

/**
 * An implementation of the {@link CodecInputOutput} interface that use socket.
 */
public class SocketCodecInputOutput implements CodecInputOutput, Recyclable {
    private Object input;

    private Object output;

    private Socket socket;

    private Reference<? extends Recyclable> recyclableReference;

    /** Default constructor. */
    public SocketCodecInputOutput() {
    }

    /**
     * @param socket
     *            a socket.
     */
    public SocketCodecInputOutput(Socket socket) {
        this.socket = socket;
    }

    /**
     * @return Returns the socket.
     */
    final Socket getSocket() {
        return socket;
    }

    /**
     * @param socket
     *            The socket to set.
     */
    final void setSocket(Socket socket) {
        this.socket = socket;
    }

    // ---------------------------------------------------------------------------
    // Implementation of the CondecInputOutput interface
    // ---------------------------------------------------------------------------

    /**
     * @see CodecInputOutput#getInput()
     */
    public Object getInput() throws IOException {
        if (input == null) {
            input = socket.getInputStream();
        }
        return input;
    }

    /**
     * @see CodecInputOutput#setInput(Object)
     */
    public void setInput(Object input) {
        this.input = input;
    }

    /**
     * @see CodecInputOutput#getOutput()
     */
    public Object getOutput() throws IOException {
        if (output == null) {
            output = socket.getOutputStream();
        }
        return output;
    }

    /**
     * @see CodecInputOutput#setOutput(Object)
     */
    public void setOutput(Object output) {
        this.output = output;
    }

    /**
     * @see CodecInputOutput#close()
     */
    public void close() throws IOException {
        if (input != null && input instanceof InputStream)
            ((InputStream) input).close();
        if (output != null && output instanceof OutputStream)
            ((OutputStream) output).close();
        if (socket != null)
            socket.close();
    }

    // ---------------------------------------------------------------------------
    // Implementation of the Recyclable interface
    // ---------------------------------------------------------------------------

    /**
     * @see org.objectweb.dream.pool.Recyclable#recycle()
     */
    public void recycle() {
        try {
            this.close();
        } catch (IOException e) {
            // silent close
        }
        input = null;
        output = null;
        socket = null;
    }

    /**
     * @see Recyclable#setReference(Reference)
     */
    public void setReference(Reference<? extends Recyclable> reference) {
        this.recyclableReference = reference;
    }

    /**
     * @see Recyclable#getReference()
     */
    public Reference<? extends Recyclable> getReference() {
        return recyclableReference;
    }
}
